@@ -143,11 +143,10 @@ def force_unicode(s):
     if isinstance(s, text_type):
         return s
 
-    last_exc = None
     # Try some default encodings:
     try:
         return s.decode("utf-8")
-    except UnicodeDecodeError as exc:
+    except UnicodeDecodeError:
         pass
 
     try:
@@ -159,7 +158,10 @@ def force_unicode(s):
         # Try chardet, if available
         encoding = chardet.detect(s)["encoding"]
         if encoding is not None:
-            return s.decode(encoding)
+            try:
+                return s.decode(encoding, 'replace')
+            except LookupError:
+                pass
 
     return s.decode('latin1', 'replace')
 
