@@ -69,12 +69,11 @@ def repo_list():
         ]
 
     if order_by == "name":
-        sort_key = lambda repo: repo.namespaced_name
+        def sort_key(repo):
+            return repo.namespaced_name
     else:
-        sort_key = lambda repo: (
-            -(repo.fast_get_last_updated_at() or -1),
-            repo.namespaced_name,
-        )
+        def sort_key(repo):
+            return -(repo.fast_get_last_updated_at() or -1), repo.namespaced_name
 
     repos = sorted(repos, key=sort_key)
     invalid_repos = sorted(invalid_repos, key=lambda repo: repo.namespaced_name)
@@ -110,7 +109,11 @@ def _get_repo_and_rev(repo, namespace=None, rev=None, path=None):
     if rev is None:
         rev = repo.get_default_branch()
         if rev is None:
-            raise NotFound("Empty repository")
+            rev = "HEAD"
+            try:
+                repo.get_commit("HEAD")
+            except KeyError:
+                raise NotFound("No commits yet")
 
     i = len(rev)
     while i > 0:
